"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var LINEBot = require("@line/bot-sdk");
var Dotenv = require("dotenv");
var express = require("express");
var fs = require("fs-extra");
var https = require("https");
var path = require("path");
var pem = require("pem");
var BotServer = /** @class */ (function () {
    function BotServer(options) {
        this.options = options || BotServer.getEnvOptions();
        if (!this.options.port) {
            throw new Error('Missing options.port\nPlease set PORT environment variable.');
        }
        if (!this.options.channelSecret) {
            throw new Error('Missing options.channelSecret\nPlease set CHANNEL_SECRET environment variable.');
        }
        if (!this.options.channelAccessToken) {
            throw new Error('Missing options.channelAccessToken\nPlease set CHANNEL_ACCESS_TOKEN environment variable.');
        }
        if (!this.options.key) {
            throw new Error('Missing options.key\nPlease set SSL_KEY environment variable.');
        }
        if (!this.options.cert) {
            throw new Error('Missing options.cert\nPlease set SSL_CERT environment variable.');
        }
        this.clientConfig = {
            channelAccessToken: this.options.channelAccessToken,
            channelSecret: this.options.channelSecret
        };
        this.client = new LINEBot.Client(this.clientConfig);
        this.app = express();
        this.https = https.createServer({
            cert: fs.readFileSync(this.options.cert),
            key: fs.readFileSync(this.options.key)
        }, this.app);
    }
    BotServer.generateDefaultSSLAsync = function (sslDir, days, selfSigned) {
        if (days === void 0) { days = 9999; }
        if (selfSigned === void 0) { selfSigned = true; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, fs.ensureDir(sslDir)];
                    case 1:
                        _a.sent();
                        BotServer.defaultSSLKey = path.resolve(sslDir, 'localhost.key');
                        BotServer.defaultSSLCert = path.resolve(sslDir, 'localhost.crt');
                        return [2 /*return*/, new Promise(function (resolve, reject) {
                                if (!fs.existsSync(BotServer.defaultSSLKey) || !fs.existsSync(BotServer.defaultSSLCert)) {
                                    pem.createCertificate({ days: days, selfSigned: selfSigned }, function (err, keys) {
                                        if (err) {
                                            return reject(err);
                                        }
                                        fs.writeFileSync(BotServer.defaultSSLKey, keys.serviceKey);
                                        fs.writeFileSync(BotServer.defaultSSLCert, keys.certificate);
                                        return resolve(true);
                                    });
                                }
                                else {
                                    return resolve(true);
                                }
                            })];
                }
            });
        });
    };
    BotServer.getEnvOptions = function (options) {
        Dotenv.config(options);
        return {
            cert: process.env.SSL_CERT || '',
            channelAccessToken: process.env.CHANNEL_ACCESS_TOKEN || '',
            channelSecret: process.env.CHANNEL_SECRET || '',
            key: process.env.SSL_KEY || '',
            port: (process.env.PORT) ? Number(process.env.PORT) : 443
        };
    };
    BotServer.generateEnvFile = function (fileName, options) {
        if (options === void 0) { options = {}; }
        var buffer = new Buffer('CHANNEL_ACCESS_TOKEN=' + (options.channelAccessToken || '') + '\n' +
            'CHANNEL_SECRET=' + (options.channelSecret || '') + '\n' +
            'PORT=' + (options.port || '') + '\n' +
            'SSL_KEY=' + (options.key || '') + '\n' +
            'SSL_CERT=' + (options.cert || '') + '\n');
        fs.writeFileSync(fileName, buffer);
    };
    BotServer.prototype.enableStatusEndpoint = function () {
        this.app.get('/status', function (req, res) {
            res.status(200);
            res.end();
        });
    };
    BotServer.prototype.setWebhook = function (endpoint, callback) {
        this.app.post(endpoint, LINEBot.middleware(this.clientConfig), callback);
    };
    BotServer.prototype.start = function () {
        this.https.listen(this.options.port);
    };
    BotServer.defaultSSLKey = 'ssl/localhost.key';
    BotServer.defaultSSLCert = 'ssl/localhost.crt';
    return BotServer;
}());
exports.BotServer = BotServer;
//# sourceMappingURL=bot-server.js.map